"use strict";
var alsatian_core_1 = require("../core/alsatian-core");
var tap_bark_1 = require("tap-bark");
var Package = require("../package.json");
var CliTestRunner = (function () {
    function CliTestRunner(_testRunner) {
        this._testRunner = _testRunner;
        if (!_testRunner) {
            throw new TypeError("_testRunner must not be null or undefined.");
        }
    }
    CliTestRunner.create = function () {
        var outputStream = new alsatian_core_1.TestOutputStream();
        var testRunner = new alsatian_core_1.TestRunner(outputStream);
        return new CliTestRunner(testRunner);
    };
    CliTestRunner.prototype.run = function (userArguments) {
        // if version has been requested then output the current version and exit
        if (userArguments.versionRequested) {
            process.stdout.write("alsatian version " + Package.version);
            return;
        }
        // if help has been requested then output info about using the CLI and exit
        if (userArguments.helpRequested) {
            process.stdout.write("\n\n" +
                "alsatian version " + Package.version + "\n" +
                "=========================\n" +
                "CLI options\n" +
                "=========================\n" +
                "HELP:    --help / -h                      (outputs CLI information)\n" +
                "VERSION: --version / -v                   (outputs the version of the CLI)\n" +
                "TAP:     --tap / -T                       (runs alsatian with TAP output)\n" +
                "TIMEOUT: --timeout [number] / -t [number] (sets the timeout period for tests in milliseconds - default 500)\n" +
                "\n");
            return;
        }
        // create test set from given file globs
        var testSet = alsatian_core_1.TestSet.create();
        testSet.addTestsFromFiles(userArguments.fileGlobs);
        if (userArguments.tap) {
            // if they want TAP output then just write to stdout directly
            this._testRunner.outputStream.pipe(process.stdout);
        }
        else {
            // otherwise create the tap bark reporter
            var bark = tap_bark_1.TapBark.create();
            // pipe the reporter into stdout
            this._testRunner.outputStream.pipe(bark.getPipeable()).pipe(process.stdout);
        }
        try {
            var testRunPromise = this._testRunner.run(testSet, userArguments.timeout);
            testRunPromise.catch(this._handleTestSetRunError);
        }
        catch (error) {
            this._handleTestSetRunError(error);
        }
    };
    CliTestRunner.prototype._handleTestSetRunError = function (error) {
        process.stderr.write(error.message + "\n");
        process.exit(1);
    };
    return CliTestRunner;
}());
exports.CliTestRunner = CliTestRunner;
//# sourceMappingURL=cli-test-runner.js.map