"use strict";
var promise_1 = require("../../promise/promise");
var alsatian_core_1 = require("../alsatian-core");
var test_plan_1 = require("./test-plan");
var test_set_run_info_1 = require("./test-set-run-info");
require("reflect-metadata");
var TestRunner = (function () {
    function TestRunner(outputStream) {
        // If we were given a TestOutput, use it, otherwise make one
        if (outputStream !== undefined) {
            this._outputStream = outputStream;
        }
        else {
            this._outputStream = new alsatian_core_1.TestOutputStream();
        }
    }
    Object.defineProperty(TestRunner.prototype, "outputStream", {
        get: function () {
            return this._outputStream;
        },
        enumerable: true,
        configurable: true
    });
    TestRunner.prototype.run = function (testSet, timeout) {
        var _this = this;
        var testPlan = new test_plan_1.TestPlan(testSet);
        if (testPlan.testItems.length === 0) {
            throw new Error("no tests to run.");
        }
        if (!timeout) {
            timeout = 500;
        }
        var testSetResults = new alsatian_core_1.TestSetResults();
        this._outputStream.emitVersion();
        this._outputStream.emitPlan(testPlan.testItems.length);
        return new promise_1.Promise(function (resolve, reject) {
            var testSetRunInfo = new test_set_run_info_1.TestSetRunInfo(testPlan, testSetResults, timeout);
            _this._scheduleNextTestPlanItem(testSetRunInfo, resolve);
        });
    };
    TestRunner.prototype._createResultAndRunNextTest = function (testSetRunInfo, resolve, error) {
        var testSetResults = testSetRunInfo.testSetResults;
        var currentTestFixtureResults = testSetResults
            .testFixtureResults[testSetResults.testFixtureResults.length - 1];
        var currentTestResults = currentTestFixtureResults
            .testResults[currentTestFixtureResults.testResults.length - 1];
        var testItem = testSetRunInfo.testPlanItem;
        var result = currentTestResults.addTestCaseResult(testItem.testCase.arguments, error);
        this._outputStream.emitResult(testSetRunInfo.testPlan.testItems.indexOf(testItem) + 1, result);
        this._scheduleNextTestPlanItem(testSetRunInfo, resolve);
    };
    TestRunner.prototype._scheduleNextTestPlanItem = function (testSetRunInfo, resolve) {
        var _this = this;
        var nextTestPlanIndex = testSetRunInfo.testPlan.testItems.indexOf(testSetRunInfo.testPlanItem) + 1;
        var nextTestPlanItem = testSetRunInfo.testPlan.testItems[nextTestPlanIndex];
        if (nextTestPlanItem) {
            testSetRunInfo.testPlanItem = nextTestPlanItem;
            var testSetResults = testSetRunInfo.testSetResults;
            var currentTestFixtureResults = testSetResults
                .testFixtureResults[testSetResults.testFixtureResults.length - 1];
            if (!currentTestFixtureResults || currentTestFixtureResults.fixture !== nextTestPlanItem.testFixture) {
                this._outputStream.emitFixture(nextTestPlanItem.testFixture);
                currentTestFixtureResults = testSetResults.addTestFixtureResult(nextTestPlanItem.testFixture);
            }
            var currentTestResults = currentTestFixtureResults
                .testResults[currentTestFixtureResults.testResults.length - 1];
            if (!currentTestResults || currentTestResults.test !== nextTestPlanItem.test) {
                currentTestResults = currentTestFixtureResults.addTestResult(nextTestPlanItem.test);
            }
            nextTestPlanItem.run(testSetRunInfo.timeout)
                .then(function (testResults) {
                _this._createResultAndRunNextTest(testSetRunInfo, resolve, testResults.error);
            })
                .catch(function (error) {
                _this._createResultAndRunNextTest(testSetRunInfo, resolve, error);
            });
        }
        else {
            resolve(testSetRunInfo.testSetResults);
            this._outputStream.end();
        }
    };
    ;
    return TestRunner;
}());
exports.TestRunner = TestRunner;
//# sourceMappingURL=test-runner.js.map